import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;


public class DialCanvas extends ScaleCanvas implements KeyListener, MouseListener, MouseMotionListener {
    Manager M;
    int MODE;
    Point CURSOR;
    boolean DRAG;
    Complex[] PLOT=new Complex[100000];
    Color[] COL=new Color[100000];
    ListenSquare ONE;
    int COUNT;

     public DialCanvas() {
	 addKeyListener(this);
	 addMouseListener(this);
	 addMouseMotionListener(this);
	 setScales(300,300,200);
	 SOURCE=new Complex(1,0);
	 CURSOR=new Point();
	 DRAG=false;
	 COUNT=0;
	 MODE=1;
	 ONE=new ListenSquare(0,0,20,20);
	 SOURCE=new Complex(0,-1);
     }


   public void paint(Graphics g2) {
      Graphics2D g=(Graphics2D) g2;
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      drawBG(g);
      drawDial(g);
      drawDiagonals(g);
      drawCrossRatio(g);
      drawHyperbolicStuff(g);
      ONE.render(g,Color.blue);
      drawTest(g);
   }


    public void drawTest(Graphics2D g) {
	if(M.C.DIAL.L[6].on==0) return;
	Color CC=new Color(0,0,255,30);
	int mode=M.C.MOTION.mode;
	if(mode<2) return;
	int k=(mode+1)%4;
	double[] list={1,-1,2,-2,.5,-.5};
	for(int a=0;a<6;++a) {
	PolygonWrapper X=new PolygonWrapper(SquareLine.getRectangle(list[a],M.P.S));
	Complex[] Z=new Complex[3];
	for(int j=0;j<3;++j) {
	    Complex z=new Complex(Math.random(),Math.random());
	    PolygonWrapper POLY=new PolygonWrapper(M.P.POLY.z);
	    POLY=POLY.swivel(k,X.z[(k+1)%4],z);
	    PolygonWrapper R1=POLY.diagonal(0);
   	    PolygonWrapper R2=POLY.diagonal(1);
	    Segment[] S=new Segment[4];
	    for(int i=0;i<4;++i) S[i]=new Segment(R1.z[i],R2.z[i]);
	    Z[j]=hypCenter(S,false);
	}
	Complex cen=Circle.center(Z[0],Z[1],Z[2]);
	double rad=Circle.radius(Z[0],Z[1],Z[2]);
	fillPoint(g,cen,rad,CC,500);
	}
    }

    

    /**This draws the background*/
    public void drawBG(Graphics2D g) {
	Color C=new Color(0,170,170);
	g.setColor(C);
        g.fillRect(0,0,getWidth(),getHeight()); 

    }

    /**This draws the cross ratio of the slopes of the 4 lines*/
    
    public void drawCrossRatio(Graphics2D g) {
        double x=Segment.crossRatio(M.P.S);
	Double X=Double.valueOf(x);
	g.setColor(Color.white);
	g.drawString(X.toString(),55,18);
    }
        
    /**This computes the hyperbolic center of the 4 lines.
       This is the point representing the relation between
       aspect ratio and slope.*/
    
    public Complex hypCenter(Segment[] S,boolean record) {
	double[][] m=DialConversions.aspectToSlopeMatrixInverse(S);
	Complex z=new Complex(0,1);
	z=DialConversions.act(m,z);
	z=HyperbolicGeometry.planeToDisk(z);
      	if((record==true)&&(M.C.DIAL.L[4].on==1)) record(z,Color.white);
	return z;
    }

    public Complex hypAnchor() {
	Complex[] z1=M.P.R1.z;
	Complex[] z2=M.P.R2.z;
	Complex w1=Complex.minus(z1[2],z1[0]);
	Complex w2=Complex.minus(z2[2],z2[0]);
	Complex u1=Complex.divide(w1,w2);
	u1=new Complex(-u1.y,u1.x);
      	u1=HyperbolicGeometry.planeToDisk(u1);
       	if(M.C.DIAL.L[5].on==1) record(u1,Color.black);
	return u1;
    }

    public void record(Complex z,Color C) {
        PLOT[COUNT]=new Complex(z);
	COL[COUNT]=C;
	++COUNT;
    }


    /**All the hyperbolic geometry stuff*/

    public void drawHyperbolicStuff(Graphics2D g) {
	int[] on=new int[6];
	for(int i=0;i<6;++i) {
	    on[i]=M.C.DIAL.L[i].on;
	}
	if(on[2]==1) drawHyperbolicGeodesic(g);
	if(on[0]==1) drawHyperbolicCenter(g);
	if(on[1]==1) drawHyperbolicAnchor(g);
	if((on[4]==1)||(on[5]==1)) drawHyperbolicRecord(g);
    }
    
    /**draws the hyperbolic geodesic connecting the
       aspect ratio blow up points.  These are the
       points on the ideal boundary representing  
       the aspect ratios of the rectangles at infinity
       for the two lines.  The two blowup points are the endpoints of 
       the geodesics through hypCenter and hypAnchor*/
    
    public void drawHyperbolicGeodesic(Graphics2D g) {
	double[] b=SquareLine.getBlowups(M.P.S);
	Complex[] z=new Complex[2];
	for(int i=0;i<2;++i) z[i]=new Complex(b[i],0);
	Path2D.Double gp=HyperbolicGeometry.join(z[0],z[1],500,false);
	gp=transform(gp);
	g.setColor(M.C.DIAL.M[2].C);
	g.draw(gp);
    }

    public void drawHyperbolicCenter(Graphics2D g) {
	Complex z1=hypCenter(M.P.S,true);
	fillPoint(g,z1,.02,M.C.DIAL.M[0].C,8);
    }

    
    public void drawHyperbolicAnchor(Graphics2D g) {
	Complex u1=hypAnchor();
	fillPoint(g,u1,.02,M.C.DIAL.M[1].C,8);
    }


    
    public void drawHyperbolicRecord(Graphics2D g) {
	for(int i=0;i<COUNT;++i) {
	    fillPoint(g,PLOT[i],.005,COL[i],8);
	}
    }

    
    
    

    /**MODE=0: slope
       MODE=1: aspect ratio*/
    
    public void drawDial(Graphics2D g) {
	double a=getAspectValue(M.P.S);
	double s=DialConversions.aspectToSlope(M.P.S,a);
	Complex z=DialConversions.realToComplex(s);
	Color C=new Color(110,120,130);
	if(MODE==0) C=new Color(140,0,255);
	fillPoint(g,new Complex(0,0),1.4,new Color(0,0,0),1024);
	fillPoint(g,new Complex(0,0),1.3,C,1024);
	drawBlowups2(g);
	C=new Color(110,120,130);
	if(MODE==1) C=new Color(140,0,255);
	fillPoint(g,new Complex(0,0),1.1,new Color(0,0,0),1024);
	fillPoint(g,new Complex(0,0),1,C,1024);
	drawBlowups1(g);
	Color[] D={Color.white,new Color(120,130,140)};
	Complex a1=new Complex(1.07,.05);
	Complex z1=Complex.times(z,new Complex(1.3,0));
	drawWedge(g,z1,a1,D[MODE]);
	Complex z2=DialConversions.realToComplex(a);
	a1=new Complex(1.08,.05);
	drawWedge(g,z2,a1,D[1-MODE]);
	drawRectangle(g);
    }


    public void drawRectangle(Graphics2D g) {
	double a=getAspectValue(M.P.S);
	double s=DialConversions.aspectToSlope(M.P.S,a);
	Complex z=DialConversions.realToComplex(s);
	z=Complex.sqRoot(z);
       	double b=Math.sqrt(1+a*a);
	Complex w=new Complex(a/b,1/b);
	Complex[] W=new Complex[4];
	W[0]=w;
	W[1]=new Complex(-w.x,w.y);
	W[2]=new Complex(-w.x,-w.y);
	W[3]=new Complex(+w.x,-w.y);
	z=Complex.times(z,new Complex(.5,0));
	for(int i=0;i<4;++i) W[i]=Complex.times(W[i],z);
	Path2D.Double p1=new Path2D.Double();
	for(int i=0;i<4;++i) {
	    if(i==0) p1.moveTo(W[i].x,W[i].y);
	    else p1.lineTo(W[i].x,W[i].y);
	}
	p1.closePath();
	p1=transform(p1);
	if(a>=0)  g.setColor(new Color(50,100,255,200));
	if(a<=0)  g.setColor(new Color(250,0,0,200));

   	g.fill(p1);
	g.setColor(new Color(255,60,255));
	g.draw(p1);

    }




    /**These are the little wedge-shaped markers*/
    
    public void drawWedge(Graphics2D g,Complex z0,Complex a1,Color C) {
	Complex a2=a1.conjugate();
	Path2D.Double p=new Path2D.Double();
	Complex z1=Complex.times(z0,a1);
	Complex z2=Complex.times(z0,a2);
	p.moveTo(z0.x,z0.y);
	p.lineTo(z1.x,z1.y);
	p.lineTo(z2.x,z2.y);
	p.closePath();
	p=transform(p);
	g.setColor(C);
	g.fill(p);
    }


    public Complex[] getDiagonals() {
	Segment[] S=M.P.getDiagonals();
	Complex[] z=new Complex[4];
	for(int i=0;i<2;++i) {
	    double s=S[i].slope();
            z[i]=DialConversions.realToComplex(s);
	}
        z[0]=Complex.times(z[0],new Complex(-1,0));
	double s=DialConversions.aspectToSlope(M.P.S,1);
	z[2]=DialConversions.realToComplex(s);
	s=DialConversions.aspectToSlope(M.P.S,-1);
	z[3]=DialConversions.realToComplex(s);
	return z;
    }


    /**This draws the markers where the aspect ratio
       is 0,1,-1,infinity*/

    public void drawDiagonals(Graphics2D g) {
	if(M.C.DIAL.L[3].on==0) return;
	Complex[] z0=new Complex[4];
	for(int i=0;i<4;++i) {
	    double t=1.0*i*Math.PI/2;
	    double c=Math.cos(t);
	    double s=Math.sin(t);
	    c=c;
	    s=s;
	    z0[i]=new Complex(c,s);
	}

	Complex[] z={z0[0],z0[2],z0[3],z0[1]};

	Complex[] w=getDiagonals();
	for(int i=0;i<w.length;++i) w[i]=Complex.times(w[i],new Complex(1.3,0));
	Complex a0=new Complex(1.06,.04);
	Color[] C={new Color(255,0,255),new Color(255,0,255),new Color(50,110,255),new Color(255,80,0)};

	for(int i=0;i<4;++i) {
	    drawWedge(g,w[i],a0,C[i]);
	    drawWedge(g,z[i],a0,C[i]);
	}

    }


    /**These are the places where the center of the inscribed
       rectangle goes to infinity*/
    
    public Complex[] getBlowups() {
	double[] a=SquareLine.getBlowups(M.P.S);	
	Complex[] z=new Complex[2];
	for(int i=0;i<2;++i) {
	    double s=DialConversions.aspectToSlope(M.P.S,a[i]);
            z[i]=DialConversions.realToComplex(s);
	}
	return z;
    }

    public void drawBlowups1(Graphics2D g) {
	double[] b=SquareLine.getBlowups(M.P.S);
	double[] a=new double[2];
	Complex[] z=new Complex[2];
	for(int i=0;i<2;++i) {
	  z[i]=DialConversions.realToComplex(b[i]);
	}
	drawPie(g,z[0],z[1],new Color(255,255,255,100));
    }


    /**This draws the places where the inscribed rectangles
       become infinitely large*/
    
    public void drawBlowups2(Graphics2D g) {
	double[] b=SquareLine.getBlowups(M.P.S);
	double[] a=new double[2];
	Complex[] z=new Complex[2];
	for(int i=0;i<2;++i) {
	  a[i]=DialConversions.aspectToSlope(M.P.S,b[i]);
	  z[i]=DialConversions.realToComplex(a[i]);
	  z[i]=z[i].scale(1.3);
	}
	drawPie(g,z[0],z[1],new Color(255,255,255,100));
    }
    

    public void drawPie(Graphics2D g,Complex z1,Complex z2,Color C) {
	double arg1=z1.arg();
	double arg2=z2.arg();
	double r=z1.norm();
	Path2D.Double p=new Path2D.Double();
	p.moveTo(0,0);
	for(int i=0;i<500;++i) {
	    double u=1.0*i/500;
	    double t=(1-u)*arg1+u*arg2;
	    double c=Math.cos(t);
	    double s=Math.sin(t);
	    Complex w=new Complex(r*c,r*s);
	    p.lineTo(w.x,w.y);
	}
	p.closePath();
	p=transform(p);
	g.setColor(C);
	g.fill(p);
    }



    /**This gets the aspect ratio value in both cases
       MODE=0: slope
       MODE=1: aspect ratio
    */
    
    public double getAspectValue(Segment[] S) {
	Complex z=Complex.unit(SOURCE);
	if(MODE==0) {
      	    Complex s=HyperbolicGeometry.diskToPlane(z);
	    return DialConversions.slopeToAspect(S,s.x);
	}
	if(MODE==1) {
      	    Complex s=HyperbolicGeometry.diskToPlane(z);
	    return s.x;
	}
	return 1;
    }

    public double getAspectValue2(Segment[] S) {
	double a=getAspectValue(S);
	Complex w=hypCenter(S,false);
	w=HyperbolicGeometry.diskToPlane(w);
	double[] h=HyperbolicGeometry.geodesic(w,new Complex(a,0));
	return 2*h[0]-a;
    }


    /**Mouse commands*/



    public void mouseClicked(MouseEvent e) { 
	MouseData J=MouseData.process(e);
	if(ONE.inside(J.X)==1) {
	    SOURCE=new Complex(0,-1);
	    M.repaint();
	    return;
	}
        if(J.mode==1)  scaleUp(J.X,0);
        if(J.mode==3)  scaleUp(J.X,1);	
        if(J.mode==2)  {
           SOURCE=unTransform(J.X);
	}
	M.repaint();
    }

    public void doMouseClick(int mode) { 
        if(mode==1)  scaleUp(CURSOR,0);
        if(mode==3)  scaleUp(CURSOR,1);	
        if(mode==2)  {
           SOURCE=unTransform(CURSOR);
	}
	M.repaint();
    }

    public void mousePressed(MouseEvent e) { 
	MouseData J=MouseData.process(e);
        if(J.mode==2)  {
           SOURCE=unTransform(J.X);
	   MODE=1;
	   if(Complex.norm(SOURCE)>1.2) MODE=0;
	}
    }

    public void doMousePress(int mode) { 
        if(mode==2)  {
           SOURCE=unTransform(CURSOR);
	   MODE=1;
	   if(Complex.norm(SOURCE)>1.2) MODE=0;
	}
    }

     public void mouseReleased(MouseEvent e) {	 
     }

     public void mouseEntered(MouseEvent e) {
	 requestFocus();
     }
     public void mouseExited(MouseEvent e) {}   

     public void mouseMoved(MouseEvent e) {
	MouseData J=MouseData.process(e);
	CURSOR=J.X;
	if(DRAG==true) SOURCE=unTransform(CURSOR);
	M.repaint();
     }
 
     public void mouseDragged(MouseEvent e) {
	MouseData J=MouseData.process(e);
        if(J.mode==2)  SOURCE=unTransform(J.X);
	M.repaint();
     }


    public void keyPressed(KeyEvent e) {
	char ch=e.getKeyChar();
	if(ch=='x') DRAG=true;
 	int test=-1;
       	if(ch=='z') test=1;   
       	if(ch=='x') test=2;   
       	if(ch=='c') test=3; 
       	doMousePress(test); 
       	M.repaint();
    }

    public void keyReleased(KeyEvent e) {
	DRAG=false;
    }

    public void keyTyped(KeyEvent e) {
	char ch=e.getKeyChar();
       	int test=-1;
       	if(ch=='z') test=1;   
       	if(ch=='x') test=2;   
       	if(ch=='c') test=3; 
       	doMouseClick(test); 
       	M.repaint();
    }

}


