import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;
import java.awt.image.*;


/**This is used to record the positions of points when you click.
   Usually this class has more stuff in it, like complex arithmetic.*/

public class HyperbolicGeometry {


    public static Complex initial() {
	return new Complex(0,1);
    }


    /**This maps the standard point to the unit disk*/

    public static Complex planeToDisk(Complex z) {
	Complex w=Complex.divide(new Complex(z.x,z.y-1),new Complex(z.x,z.y+1));
	return w;
    }

    public static Complex diskToPlane(Complex w) {
	Complex z=Complex.divide(new Complex(1+w.x,w.y),new Complex(1-w.x,-w.y));
	z=Complex.times(z,new Complex(0,1));
	return z;
    }


    /**the point x is the center of the upper half plane
       geodesic through a and b.  The point r is the
       Euclidean radius of the geodesic*/
    
    public static double[] geodesic(Complex a,Complex b) {
	double x=(a.x*a.x-b.x*b.x +a.y*a.y-b.y*b.y)/(2*(a.x-b.x));
	Complex z=new Complex(x,0);
	double r=Complex.dist(z,a);
	double[] D={x,r};
	return D;
    }

    public static Path2D.Double join(Complex z1,Complex z2,int detail,boolean invert) {
	Complex[] Z=join0(z1,z2,detail);
	Path2D.Double gp=new Path2D.Double();
	for(int i=0;i<Z.length;++i) {
	    Complex w=Z[i];
	    if(invert==true) {
	      w=w.inverse();
	      w=w.conjugate();
	    }
	    if(i==0) gp.moveTo(w.x,w.y);
	    if(i!=0) gp.lineTo(w.x,w.y);
	}
	return gp;
    }

    public static Complex[] join0(Complex z1,Complex z2,int detail) {
	if(Math.abs(z1.x-z2.x)<.000000001) return joinVertical(z1,z2,detail);
	double[] D=geodesic(z1,z2);
	Complex DD=new Complex(D[0],0);
	double a1=myAtan(Complex.minus(z1,DD));
	double a2=myAtan(Complex.minus(z2,DD));
	Complex[] Z=new Complex[detail];
	Path2D.Double gp=new Path2D.Double();
	for(int i=0;i<detail;++i) {
	    double t=1.0*i/detail;
	    double x=(1-t)*a1+t*a2;
	    double c=D[0]+D[1]*Math.cos(x);
	    double s=D[1]*Math.sin(x);
	    Complex w=new Complex(c,s);
	    w=planeToDisk(w);
	    Z[i]=new Complex(w);
	}
	return Z;
    }

    public static Complex[] joinVertical(Complex z1,Complex z2,int detail) {
	Complex[] Z=new Complex[detail];
	for(int i=0;i<detail;++i) {
	    double t=1.0*i/detail;
	    Complex w=new Complex((1-t)*z1.x+t*z2.x,(1-t)*z1.y+t*z2.y);
	    Z[i]=planeToDisk(w);
	}
	return Z;
    }


    public static double myAtan(Complex z) {
	double d=Math.atan2(z.y,z.x);
	if(d<0) d=d+2*Math.PI;
	return d;
    }




}
