import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.awt.geom.*;

public class PolygonWrapper {
    Complex[] z=new Complex[4];

    public PolygonWrapper() {
	for(int i=0;i<4;++i) z[i]=new Complex();
    }

    public PolygonWrapper(Complex[] Z) {
	for(int i=0;i<4;++i) z[i]=new Complex(Z[i]);
    }

    public Path2D.Double toPath() {
	Path2D.Double gp=new Path2D.Double();
	gp.moveTo(z[0].x,z[0].y);
	for(int i=0;i<4;++i) {
	    gp.lineTo(z[i].x,z[i].y);
	}
	gp.closePath();
	return(gp);
    }
    
    public Path2D.Double toPathEmpty() {
	Path2D.Double gp=new Path2D.Double();
	gp.moveTo(z[0].x,z[0].y);
	for(int i=0;i<5;++i) {
	    int j=i%4;
	    gp.lineTo(z[j].x,z[j].y);
	}
	gp.closePath();
	return(gp);
    }

    public Complex center() {
	Complex w=new Complex();
	for(int i=0;i<4;++i) {
	    w=Complex.plus(w,this.z[i]);
	}
	w.x=w.x/4;
	w.y=w.y/4;
	return(w);
    }

    public double aspectRatio() {
	double d1=Complex.dist(z[0],z[1]);
	double d2=Complex.dist(z[2],z[1]);
	double d=d2/d1;
	return d;
    }

    public int nearestPoint(Complex w) {
	double min=99999;
	int index=0;
	for(int i=0;i<4;++i) {
	    double test=Complex.dist(w,z[i]);
	    if(test<min) {
		index=i;
		min=test;
	    }
	}
	return index;
    }

    public PolygonWrapper translate(Complex Z) {
	Complex w=this.center();
	w=Complex.minus(Z,w);
	PolygonWrapper P=new PolygonWrapper();
	for(int i=0;i<4;++i) P.z[i]=Complex.plus(z[i],w);
	return P;
    }

    public void print() {
	System.out.println("---------");
	for(int i=0;i<4;++i) z[i].print();
    }


    public PolygonWrapper swivel(int k,Complex pivot,Complex w) {
	Segment[] S=new Segment[4];
	for(int i=0;i<4;++i) {
	    int j=(i+1)%4;
	    S[i]=new Segment(z[i],z[j]);
	}
	S[k].z[0]=new Complex(pivot);
	S[k].z[1]=new Complex(w);



	Complex z01=Segment.cross(S[0],S[1]);
	Complex z12=Segment.cross(S[1],S[2]);
	Complex z23=Segment.cross(S[2],S[3]);
	Complex z30=Segment.cross(S[3],S[0]);
	Complex[] Z={z30,z01,z12,z23};
	PolygonWrapper ZZ=new PolygonWrapper(Z);
	return ZZ;
    }

    public static Complex rotate(Complex a,Complex b,Complex u) {
	Complex z=Complex.minus(a,b);
	z=Complex.times(z,u);
	z=Complex.plus(z,b);
	return z;
    }

    public PolygonWrapper diagonal(int k) {
	PolygonWrapper P=new PolygonWrapper();
	P.z[(0+k)%4]=new Complex(z[0+k]);
	P.z[(1+k)%4]=new Complex(z[0+k]);
	P.z[(2+k)%4]=new Complex(z[2+k]);
	P.z[(3+k)%4]=new Complex(z[2+k]);
	return P;
    }

    public static PolygonWrapper fromDiagonals(PolygonWrapper A,PolygonWrapper B) {
	PolygonWrapper P=new PolygonWrapper();
	P.z[0]=new Complex(A.z[0]);
	P.z[1]=new Complex(B.z[0]);
	P.z[2]=new Complex(A.z[2]);
	P.z[3]=new Complex(B.z[2]);
	return P;
    }
    

}
