import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;

/**This class finds rectangles inscribed in triples and quadruples of lines.*/

public class SquareLine {


    /**This is the first of the main routines.  It gets the
       rectangle of aspect ratio r inscribed in the line segments*/

    public static Complex[] getRectangle(double r,Segment[] S) {
	Complex[][] Z=convert(S);
	double[] R={r,1/r,r,1/r};
	Complex[] Q=new Complex[4];
	for(int i=0;i<4;++i) {
	    Q[i]=getPoint(R[i],Z);
	    Z=rotate(Z);
	}
	return Q;
    }

    /**This gets a the point of the rectangle of aspect ratio r on line 0.*/

    public static Complex getPoint(double r,Complex[][] Z) {
	double[] L=getCoeff(Z);
	double nn=L[0]+L[1]*r+L[2]*r*r;
        double dd=L[3]+L[4]*r+L[5]*r*r;
	double X= nn/dd;
	double[] l=lineForm(Z[0]);
	double Y=l[0]*X+l[1];
	return new Complex(X,Y);
    }


    /**This gets the conjugate ratio*/
    public static double getConjugateRatio(int k,double r0,Segment[] S) {
	Complex[][] Z=convert(S);
	for(int i=0;i<k;++i) {
	    Z=rotate(Z);
	}
	double r=r0;
      	if(k%2==1) r=1/r;
	double rr=getConjugateRatio(r,Z);
	if(k%2==1) rr=1/rr;
	return rr;
    }
    
    public static double getConjugateRatio(double r,Complex[][] Z) {
	double[] L=getCoeff(Z);
	double nn=L[0]+L[1]*r+L[2]*r*r;
        double dd=L[3]+L[4]*r+L[5]*r*r;
	double X= nn/dd;
	double aa=L[2]-X*L[5];
	double bb=L[1]-X*L[4];
	double cc=L[0]-X*L[3];
	double[] rr=QuadraticSolver.main(aa,bb,cc,.0000001);
	double test=Math.abs(rr[0]-r);
	if(test<.00000001) return rr[1];
	return rr[0];
    }


    /**The point usually varies monotonically with
       the aspect ratio parameter, but sometimes
       there are turn-around points. This routine
       gets these points*/

    public static Complex[] getTurnsX(Segment[] S) {
	Complex[][] Z=convert(S);
	Complex[] TURN=new Complex[8];
	int count=0;
	for(int i=0;i<4;++i) {
	    Complex[] LIST=getTurnSingle(Z);
	    if(LIST!=null) {
		for(int j=0;j<LIST.length;++j) {
		    TURN[count]=LIST[j];
		    ++count;
		}
	    }
	    Z=rotate(Z);
	}
	if(count==0) return null;
	Complex[] TURN2=new Complex[count];
	for(int i=0;i<count;++i) TURN2[i]=new Complex(TURN[i]);
	return TURN2;
    }

    public static Complex[] getTurns(int k,Segment[] S) {
	Complex[][] Z=convert(S);
	for(int i=0;i<k;++i) Z=rotate(Z);
	Complex[] TURN=new Complex[2];
	Complex[] LIST=getTurnSingle(Z);
	return LIST;
    }

    /**This gets the stagnation points for line 0*/

    public static Complex[] getTurnSingle(Complex[][] Z) {
	double[] L=getCoeff(Z);
	double aa=1*(L[2]*L[4]-L[1]*L[5]);
	double bb=2*(L[2]*L[3]-L[0]*L[5]);
	double cc=1*(L[1]*L[3]-L[0]*L[4]);
	double[] X= QuadraticSolver.main(aa,bb,cc,.0000001);
	if(X==null) return null;
	Complex[] H=new Complex[X.length];
	for(int i=0;i<X.length;++i) {
	   double r=X[i];
  	   double nn=L[0]+L[1]*r+L[2]*r*r;
           double dd=L[3]+L[4]*r+L[5]*r*r;
	   double xx=nn/dd;
	   double[] l=lineForm(Z[0]);
	   double yy=l[0]*xx+l[1];
	   H[i]=new Complex(xx,yy);
	}
	return H;
    }

    /**Gets the positive aspect ratio values where the
       rectangles blow up*/

    public static double[] getBlowups(Segment[] S) {
	Complex[][] Z=convert(S);
	double[] L=getCoeff(Z);
	double[] q=QuadraticSolver.main(L[5],L[4],L[3],.0000001);
	return q;
    }

    /**This routine tells if 2 rectangles, of sepect ratio r1 and r2 nd
       supported by the same lines, are continuously connected by
       rectangles supported to the same line.  The routine implements
       Sturm sequences to decide whether the denominator of the  
       associated rational function has a root between r1 and r2.**/

    public static boolean connect(double r1,double r2,Segment[] S,double tol) {
	Complex[][] Z=convert(S);
	double[] L=getCoeff(Z);
	double a=L[5];
	double b=L[4];
	double c=L[3];
	if(Math.abs(a)<tol) {
	    double x1=b*r1+c;
	    double x2=b*r2+c;
	    if(x1*x2<0) return false;
	    return true;
	}
	double x1=a*r1*r1+b*r1+c;
	double y1=2*a*r1+b;
	double z1=-c+(b*b)/(4*a);
	double x2=a*r2*r2+b*r2+c;
	double y2=2*a*r2+b;
	double z2=z1;
	int count1=signChanges(x1,y1,z1);
	int count2=signChanges(x2,y2,z2);
	if(count1!=count2) return false;
	return true;
    }

    /**Counts the sign changes in the sequence x,y,z. This
       is for the Sturm sequences.*/

    public static int signChanges(double x,double y,double z) {
	int count=0;
	if(x*y<0) ++count;
	if(y*z<0) ++count;
	return count;
    }

    /**Converts a list of segments to complex numbers*/
    public static Complex[][] convert(Segment[] S) {
	Complex[][] Z=new Complex[4][2];
	for(int i=0;i<4;++i) {
	    Z[i][0]=S[i].z[0];
	    Z[i][1]=S[i].z[1];
	}
	return Z;
    }

    /**This cycles through the points*/

    public static Complex[][] rotate(Complex[][] Z) {
	Complex[][] W=new Complex[4][2];
	for(int i=0;i<4;++i) {
	    int j=(i+1)%4;
	    W[i][0]=new Complex(Z[j][0]);
	    W[i][1]=new Complex(Z[j][1]);
	}
	return W;
    }

    /**This gets the coefficients for the bi-quadratic formula*/

    public static double[] getCoeff(Complex[][] Z) {
	double[][] x=new double[4][2];
	for(int i=0;i<4;++i) x[i]=lineForm(Z[i]);
	double[] m={x[0][0],x[1][0],x[2][0],x[3][0]};
	double[] b={x[0][1],x[1][1],x[2][1],x[3][1]};
	double m0=m[0];
	double b0=b[0];
	double m1=m[1];
	double b1=b[1];
	double m2=m[2];
	double b2=b[2];
	double m3=m[3];
	double b3=b[3];
	double S0=-(b0-b3)*(m1-m2);
	double S1=-b0+b1-b2+b3-b0*m1*m2+b3*m1*m2+b0*m1*m3-b2*m1*m3-b0*m2*m3+b1*m2*m3;
        double S2=-(b0-b1)*(m2-m3);
        double U0=+(m1-m2)*(m0-m3);
	double U1=+m0-m1+m2+m0*m1*m2-m3-m0*m1*m3+m0*m2*m3-m1*m2*m3;
	double U2=+(m0-m1)*(m2-m3);
	double[] LIST={S0,S1,S2,U0,U1,U2};
	return LIST;
    }


    /**Puts the line in y=mx+b form.*/

    public static double[] lineForm(Complex[] Z) {
	double m=(Z[1].y-Z[0].y)/(Z[1].x-Z[0].x);
	double t=Z[0].x/(Z[0].x-Z[1].x);
	double b=(1-t)*Z[0].y+t*Z[1].y;
	double[] X={m,b};
	return X;
    }

}
